/*
Playing Highligh Module for DiscordJS
Author: Flisher (andre@jmle.net)
Version 1.6.0

##History:
1.6.0 NodeJS Linting
1.5.1 Reworked console message to my new standards  
1.5.0 Added multiple explicit variable declaration  
1.4.0 Switched code to all fit within the module.exports section  
1.3.0 Added async and await in some function to prevent Discord API spam  
1.2.0 Improved collection.find to the new format (remove deprecation warning in discord.js v11.4 and prepare for v12)  
1.0.1 Fixed instructions  
1.0.0 Initial publish  

// Todo: 
	Add randomness in the minutes for the cron task
	Add a limit of actions pe rminutes
	Add muti roles capability per server

*/

module.exports = async (bot, options)  => {
    let cron = require('node-cron');
	let Ready = false

	let description =  {
        name: `discord-playing`,
        filename: `playing.js`,
        version: `1.6.0`
    }
	
	// Add check on startup and intialize the Ready Value
	bot.on("ready", () => {
		Ready = true;
		Check(bot, options);
	});

    // Add a Cron job every minutes
    let jobPlayingCheck = new cron.schedule('* * * * *', function(){			// check every minutes
		if (!Ready) return;
        Check(bot, options);
    });   

	async function Check(bot, options) {
		if (!Ready) return;
		Ready = false

		if (options && options.live) {
			// Single Server Config, will default to first guild found in the bot
			let guild = bot.guilds.first();
			GamingLive(guild, options)
			GamingNotLive(guild, options)
		}
		else {
			// Multi-Servers Config
			for(let key in options) {
				// check that guild is loaded			
				let guild = bot.guilds.get(key);
				if (guild) {
					await GamingLive(guild, options[key])
					await GamingNotLive(guild,  options[key])
				}
				else {
					console.log(`${description.name} -> Check - Bot isn't connected to Guild "${key}"`)
				}
			}
		}
		Ready = true
	}

	async function GamingLive(guild, options) {
		let bypass
		// Check if the bot can manage Roles for this guild
		if (guild.me.hasPermission("MANAGE_ROLES")) {
			// Loop trough presence to find streamers (type 1)
			let presences = guild.presences;
			if (presences) {
				presences.forEach(async function(element, key){
					if (element.game) {
						if (typeof(element.game.type) != undefined) {
							if (element.game.type === 0) {
								if (options.games[element.game.name]) {
									// key = user id
									let member = guild.members.get(key)
									// Check if the "live" Role exist
									if (guild.roles.find(val => val.name === options.live)) {
											val => val.username === 'Bob'
										// Check if the position of the "live" role is managable by the bot
										if (guild.me.highestRole.position >= guild.roles.find(val => val.name === options.live).position) {
											// Check if there is a Role required in the configuration
											bypass = false;
											if (typeof(options.required) === "undefined") {
												// If there is no required role, bypass
												bypass = true;
											}
											else {
												// Check if the required role exist and log an error message if missing
												if (!guild.roles.find(val => val.name === options.required)) 
												{
													console.log(`${description.name} -> GamingLive - REQUIRED Role "${options.live}" doesn't exist on Guild "${guild.name}" (${guild.id})`);
												}

											}
											if (bypass || (member.roles.find(val => val.name === options.required))) {
												// Check if the member doesn't already have the live role
												if (!(member.roles.find(val => val.name === options.live))) {
													await member.addRole(guild.roles.find(val => val.name === options.live)).catch(console.error);
												}
											}								
										}
										else {
											console.log(`${description.name} -> GamingLive - LIVE Role "${options.live}" is higher than the bot highest permission on Guild "${guild.name}" (${guild.id})`);
										}
									}
									else {
										console.log(`${description.name} -> GamingLive - LIVE Role "${options.live}" doesn't exist on Guild "${guild.name}" (${guild.id})`);
									}



								}							
							}
						}
					}
				});
			}
		} 
		else {
			console.log(`${description.name} -> GamingLive - Bot doesn't have "MANAGE_ROLES" permission on Guild "${guild.name}" (${guild.id})`);
		}
	}

	async function GamingNotLive(guild, options) {
		// Check if the bot can manage Roles for this guild
		if (guild.me.hasPermission("MANAGE_ROLES")) {
			// Check if the live Role exist
			if (guild.roles.find(val => val.name === options.live)) {
				// Check if the position of the "live" role is managable by the bot
				if (guild.me.highestRole.position >= guild.roles.find(val => val.name ===  options.live).position) {
					// Loop les membres du Role Live pour ret
					let streamers = guild.roles.find(val => val.name ===  options.live).members
					streamers.forEach(async function(member, key){
						let stillPlaying = 0;
						let presence = member.guild.presences.get(key);
						if (presence) {
							if (presence.game) {
								if (typeof(presence.game.type) != undefined)  {
									if (presence.game.type === 0) {
										if (options.games[presence.game.name]) {
											stillPlaying = 1;
										}
									}
								}
							}
							if (stillPlaying == 0)
							{
								await member.removeRole(guild.roles.find(val => val.name ===  options.live)).catch(console.error);
							}
						}
					});
				}
				else {
					console.log(`${description.name} -> GamingNotLive - LIVE Role "${options.live}" is higher than the bot highest permission on Guild "${guild.name}" (${guild.id})`);
				}
			}
			else {
				console.log(`${description.name} -> GamingNotLive - LIVE Role "${options.live}" doesn't exist on Guild "${guild.name}" (${guild.id})`);
			}
		}
		else {
			console.log(`${description.name} -> GamingNotLive - Bot doesn't have "MANAGE_ROLES" permission on Guild "${guild.name}" (${guild.id})`);
		}
	}

}
